@@ -31,12 +31,14 @@ lws_now_usecs(void)
 	if (clock_gettime(CLOCK_MONOTONIC, &ts))
 		return 0;
 
-	return (ts.tv_sec * LWS_US_PER_SEC) + (ts.tv_nsec / LWS_NS_PER_US);
+	return (((lws_usec_t)ts.tv_sec) * LWS_US_PER_SEC) +
+			((lws_usec_t)ts.tv_nsec / LWS_NS_PER_US);
 #else
 	struct timeval now;
 
 	gettimeofday(&now, NULL);
-	return (now.tv_sec * 1000000ll) + now.tv_usec;
+	return (((lws_usec_t)now.tv_sec) * LWS_US_PER_SEC) +
+			(lws_usec_t)now.tv_usec;
 #endif
 }
 
