@@ -25,6 +25,33 @@
 #include "core/private.h"
 
 
+int
+_lws_plat_service_forced_tsi(struct lws_context *context, int tsi)
+{
+	struct lws_context_per_thread *pt = &context->pt[tsi];
+	int m, n;
+
+	lws_service_flag_pending(context, tsi);
+
+	/* any socket with events to service? */
+	for (n = 0; n < (int)pt->fds_count; n++) {
+		if (!pt->fds[n].revents)
+			continue;
+
+		m = lws_service_fd_tsi(context, &pt->fds[n], tsi);
+		if (m < 0)
+			return -1;
+		/* if something closed, retry this slot */
+		if (m)
+			n--;
+	}
+
+	lws_service_do_ripe_rxflow(pt);
+
+	return 0;
+}
+
+
 LWS_EXTERN int
 _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 {
@@ -57,27 +84,11 @@ _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 		pt->service_tid_detected = 1;
 	}
 
-	if (timeout_ms < 0) {
-		if (lws_service_flag_pending(context, tsi)) {
-			/* any socket with events to service? */
-			for (n = 0; n < (int)pt->fds_count; n++) {
-				int m;
-				if (!pt->fds[n].revents)
-					continue;
-
-				m = lws_service_fd_tsi(context, &pt->fds[n], tsi);
-				if (m < 0)
-					return -1;
-				/* if something closed, retry this slot */
-				if (m)
-					n--;
-			}
-		}
-		return 0;
-	}
-
-	/* force a default timeout of 23 days */
-	timeout_ms = 2000000000;
+	if (timeout_ms < 0)
+		timeout_ms = 0;
+	else
+		/* force a default timeout of 23 days */
+		timeout_ms = 2000000000;
 	timeout_us = ((lws_usec_t)timeout_ms) * LWS_US_PER_MS;
 
 	if (context->event_loop_ops->run_pt)
@@ -113,21 +124,18 @@ _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 	/*
 	 * is there anybody with pending stuff that needs service forcing?
 	 */
-	if (!lws_service_adjust_timeout(context, 1, tsi)) {
-		/* -1 timeout means just do forced service */
-		_lws_plat_service_tsi(context, -1, pt->tid);
-		/* still somebody left who wants forced service? */
-		if (!lws_service_adjust_timeout(context, 1, pt->tid))
-			/* yes... come back again quickly */
-			timeout_us = 0;
-	}
+	if (!lws_service_adjust_timeout(context, 1, tsi))
+		_lws_plat_service_forced_tsi(context, tsi);
 
-	if (timeout_us) {
+	/*
+	 * service pending callbakcs and get maximum wait time
+	 */
+	{
 		lws_usec_t us;
 
 		lws_pt_lock(pt, __func__);
 		/* don't stay in poll wait longer than next hr timeout */
-		us = __lws_sul_check(&pt->pt_sul_owner, lws_now_usecs());
+		us = __lws_sul_service_ripe(&pt->pt_sul_owner, lws_now_usecs());
 		if (us && us < timeout_us)
 			timeout_us = us;
 
@@ -201,10 +209,13 @@ _lws_plat_service_tsi(struct lws_context *context, int timeout_ms, int tsi)
 				lws_service_fd_tsi(context, pfd, tsi);
 			}
 		}
-	} else if (ev == WSA_WAIT_TIMEOUT) {
-		lws_service_fd(context, NULL);
-	} else if (ev == WSA_WAIT_FAILED)
+
 		return 0;
+	}
+
+	// if (ev == WSA_WAIT_TIMEOUT) { }
+	// if (ev == WSA_WAIT_FAILED)
+		// return 0;
 
 	return 0;
 }
