@@ -214,7 +214,7 @@ lws_client_socket_service(struct lws *wsi, struct lws_pollfd *pollfd,
 			lwsl_client("SOCKS password OK, sending connect\n");
 			if (socks_generate_msg(wsi, SOCKS_MSG_CONNECT, &len)) {
 socks_send_msg_fail:
-				*cce = "socks gen msg fail";
+				cce = "socks gen msg fail";
 				goto bail3;
 			}
 			conn_mode = LRS_WAITING_SOCKS_CONNECT_REPLY;
@@ -291,14 +291,17 @@ socks_reply_fail:
 		}
 
 		pt->serv_buf[13] = '\0';
-		if (strncmp(sb, "HTTP/1.0 200 ", 13) &&
-		    strncmp(sb, "HTTP/1.1 200 ", 13)) {
+		if (n < 13 || (strncmp(sb, "HTTP/1.0 200 ", 13) &&
+		    strncmp(sb, "HTTP/1.1 200 ", 13))) {
 			lwsl_err("%s: ERROR proxy did not reply with h1\n",
 					__func__);
+			/* lwsl_hexdump_notice(sb, n); */
 			cce = "proxy not h1";
 			goto bail3;
 		}
 
+		lwsl_info("%s: proxy connection extablished\n", __func__);
+
 		/* clear his proxy connection timeout */
 
 		lws_set_timeout(wsi, NO_PENDING_TIMEOUT, 0);
