@@ -1,7 +1,7 @@
 /*
  * libwebsockets-test-server - libwebsockets test implementation
  *
- * Copyright (C) 2010-2017 Andy Green <andy@warmcat.com>
+ * Written in 2010-2019 by Andy Green <andy@warmcat.com>
  *
  * This file is made available under the Creative Commons CC0 1.0
  * Universal Public Domain Dedication.
@@ -16,12 +16,17 @@
  * The test apps are intended to be adapted for use in your code, which
  * may be proprietary.  So unlike the library itself, they are licensed
  * Public Domain.
+ *
+ * Notice that the lws_pthread... locking apis are all zero-footprint
+ * NOPs in the case LWS_MAX_SMP == 1, which is the default.  When lws
+ * is built for multiple service threads though, they resolve to their
+ * pthreads equivalents.
  */
 
 #if !defined (LWS_PLUGIN_STATIC)
 #define LWS_DLL
 #define LWS_INTERNAL
-#include "../lib/libwebsockets.h"
+#include <libwebsockets.h>
 #endif
 
 #include <string.h>
@@ -31,7 +36,7 @@
 /* queue free space below this, rx flow is disabled */
 #define RXFLOW_MIN (4)
 /* queue free space above this, rx flow is enabled */
-#define RXFLOW_MAX (QUEUELEN / 3)
+#define RXFLOW_MAX ((2 * QUEUELEN) / 3)
 
 #define MAX_MIRROR_INSTANCES 3
 
@@ -52,7 +57,10 @@ struct a_message {
 
 struct mirror_instance {
 	struct mirror_instance *next;
+	lws_pthread_mutex(lock) /* protects all mirror instance data */
 	struct per_session_data__lws_mirror *same_mi_pss_list;
+	/**< must hold the the per_vhost_data__lws_mirror.lock as well
+	 * to change mi list membership */
 	struct lws_ring *ring;
 	int messages_allocated;
 	char name[30];
@@ -60,13 +68,14 @@ struct mirror_instance {
 };
 
 struct per_vhost_data__lws_mirror {
+	lws_pthread_mutex(lock) /* protects mi_list membership changes */
 	struct mirror_instance *mi_list;
 };
 
 
 /* enable or disable rx from all connections to this mirror instance */
 static void
-mirror_rxflow_instance(struct mirror_instance *mi, int enable)
+__mirror_rxflow_instance(struct mirror_instance *mi, int enable)
 {
 	lws_start_foreach_ll(struct per_session_data__lws_mirror *,
 			     pss, mi->same_mi_pss_list) {
@@ -99,7 +108,7 @@ mirror_rxflow_instance(struct mirror_instance *mi, int enable)
  * Returns 0 if oldest unchanged or 1 if oldest changed from this call.
  */
 static int
-mirror_update_worst_tail(struct mirror_instance *mi)
+__mirror_update_worst_tail(struct mirror_instance *mi)
 {
 	uint32_t wai, worst = 0, worst_tail = 0, oldest;
 	struct per_session_data__lws_mirror *worst_pss = NULL;
@@ -108,7 +117,8 @@ mirror_update_worst_tail(struct mirror_instance *mi)
 
 	lws_start_foreach_ll(struct per_session_data__lws_mirror *,
 			     pss, mi->same_mi_pss_list) {
-		wai = lws_ring_get_count_waiting_elements(mi->ring, &pss->tail);
+		wai = (uint32_t)lws_ring_get_count_waiting_elements(mi->ring,
+								&pss->tail);
 		if (wai >= worst) {
 			worst = wai;
 			worst_tail = pss->tail;
@@ -122,15 +132,14 @@ mirror_update_worst_tail(struct mirror_instance *mi)
 	lws_ring_update_oldest_tail(mi->ring, worst_tail);
 	if (oldest == lws_ring_get_oldest_tail(mi->ring))
 		return 0;
-
 	/*
 	 * The oldest tail did move on.  Check if we should re-enable rx flow
 	 * for the mirror instance since we made some space now.
 	 */
 	if (!mi->rx_enabled && /* rx is disabled */
-	    lws_ring_get_count_free_elements(mi->ring) > RXFLOW_MAX)
+	    lws_ring_get_count_free_elements(mi->ring) >= RXFLOW_MAX)
 		/* there is enough space, let's re-enable rx for our instance */
-		mirror_rxflow_instance(mi, 1);
+		__mirror_rxflow_instance(mi, 1);
 
 	/* if nothing in queue, no timeout needed */
 	if (!worst)
@@ -158,7 +167,7 @@ mirror_update_worst_tail(struct mirror_instance *mi)
 }
 
 static void
-mirror_callback_all_in_mi_on_writable(struct mirror_instance *mi)
+__mirror_callback_all_in_mi_on_writable(struct mirror_instance *mi)
 {
 	/* ask for WRITABLE callback for every wsi on this mi */
 	lws_start_foreach_ll(struct per_session_data__lws_mirror *,
@@ -168,7 +177,7 @@ mirror_callback_all_in_mi_on_writable(struct mirror_instance *mi)
 }
 
 static void
-mirror_destroy_message(void *_msg)
+__mirror_destroy_message(void *_msg)
 {
 	struct a_message *msg = _msg;
 
@@ -187,10 +196,10 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 			(struct per_vhost_data__lws_mirror *)
 			lws_protocol_vh_priv_get(lws_get_vhost(wsi),
 						 lws_get_protocol(wsi));
+	char name[300], update_worst, sent_something, *pn = name;
 	struct mirror_instance *mi = NULL;
 	const struct a_message *msg;
 	struct a_message amsg;
-	char name[300], update_worst, sent_something;
 	uint32_t oldest_tail;
 	int n, count_mi = 0;
 
@@ -203,78 +212,87 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 		 * "?mirror=xxx", will be "xxx"
 		 */
 		name[0] = '\0';
-		if (lws_get_urlarg_by_name(wsi, "mirror", name,
+		if (!lws_get_urlarg_by_name(wsi, "mirror", name,
 					   sizeof(name) - 1))
 			lwsl_debug("get urlarg failed\n");
-		lwsl_info("%s: mirror name '%s'\n", __func__, name);
+		if (strchr(name, '='))
+			pn = strchr(name, '=') + 1;
+
+		//lwsl_notice("%s: mirror name '%s'\n", __func__, pn);
 
 		/* is there already a mirror instance of this name? */
 
+		lws_pthread_mutex_lock(&v->lock); /* vhost lock { */
+
 		lws_start_foreach_ll(struct mirror_instance *, mi1,
 				     v->mi_list) {
 			count_mi++;
-			if (strcmp(name, mi1->name))
-				continue;
-			/* yes... we will join it */
-			lwsl_info("Joining existing mi %p '%s'\n", mi1, name);
-			mi = mi1;
-			break;
+			if (!strcmp(pn, mi1->name)) {
+				/* yes... we will join it */
+				mi = mi1;
+				break;
+			}
 		} lws_end_foreach_ll(mi1, next);
 
 		if (!mi) {
 
 			/* no existing mirror instance for name */
-			if (count_mi == MAX_MIRROR_INSTANCES)
+			if (count_mi == MAX_MIRROR_INSTANCES) {
+				lws_pthread_mutex_unlock(&v->lock); /* } vh lock */
 				return -1;
+			}
 
 			/* create one with this name, and join it */
 			mi = malloc(sizeof(*mi));
 			if (!mi)
-				return 1;
+				goto bail1;
 			memset(mi, 0, sizeof(*mi));
 			mi->ring = lws_ring_create(sizeof(struct a_message),
 						   QUEUELEN,
-						   mirror_destroy_message);
+						   __mirror_destroy_message);
 			if (!mi->ring) {
 				free(mi);
-				return 1;
+				goto bail1;
 			}
 
 			mi->next = v->mi_list;
 			v->mi_list = mi;
-			strcpy(mi->name, name);
+			lws_snprintf(mi->name, sizeof(mi->name) - 1, "%s", pn);
 			mi->rx_enabled = 1;
 
-			lwsl_info("Created new mi %p '%s'\n", mi, name);
+			lws_pthread_mutex_init(&mi->lock);
+
+			lwsl_notice("Created new mi %p '%s'\n", mi, pn);
 		}
 
 		/* add our pss to list of guys bound to this mi */
 
-		pss->same_mi_pss_list = mi->same_mi_pss_list;
-		mi->same_mi_pss_list = pss;
+		lws_ll_fwd_insert(pss, same_mi_pss_list, mi->same_mi_pss_list);
 
 		/* init the pss */
 
 		pss->mi = mi;
 		pss->tail = lws_ring_get_oldest_tail(mi->ring);
 		pss->wsi = wsi;
+
+		lws_pthread_mutex_unlock(&v->lock); /* } vhost lock */
 		break;
 
+bail1:
+		lws_pthread_mutex_unlock(&v->lock); /* } vhost lock */
+		return 1;
+
 	case LWS_CALLBACK_CLOSED:
 		/* detach our pss from the mirror instance */
 		mi = pss->mi;
 		if (!mi)
 			break;
 
-		/* remove our closing pss from its mirror instance list */
-		lws_start_foreach_llp(struct per_session_data__lws_mirror **,
-			ppss, mi->same_mi_pss_list) {
-			if (*ppss == pss) {
-				*ppss = pss->same_mi_pss_list;
-				break;
-			}
-		} lws_end_foreach_llp(ppss, same_mi_pss_list);
+		lws_pthread_mutex_lock(&v->lock); /* vhost lock { */
 
+		/* remove our closing pss from its mirror instance list */
+		lws_ll_fwd_remove(struct per_session_data__lws_mirror,
+				  same_mi_pss_list, pss, mi->same_mi_pss_list);
 		pss->mi = NULL;
 
 		if (mi->same_mi_pss_list) {
@@ -285,7 +303,10 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 			 * tail.  If the oldest tail moves on, this call also
 			 * will re-enable rx flow control when appropriate.
 			 */
-			mirror_update_worst_tail(mi);
+			lws_pthread_mutex_lock(&mi->lock); /* mi lock { */
+			__mirror_update_worst_tail(mi);
+			lws_pthread_mutex_unlock(&mi->lock); /* } mi lock */
+			lws_pthread_mutex_unlock(&v->lock); /* } vhost lock */
 			break;
 		}
 
@@ -293,16 +314,18 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 
 		lws_start_foreach_llp(struct mirror_instance **,
 				pmi, v->mi_list) {
-			if (*pmi != mi)
-				continue;
+			if (*pmi == mi) {
+				*pmi = (*pmi)->next;
 
-			*pmi = (*pmi)->next;
+				lws_ring_destroy(mi->ring);
+				lws_pthread_mutex_destroy(&mi->lock);
 
-			lws_ring_destroy(mi->ring);
-			free(mi);
-			break;
+				free(mi);
+				break;
+			}
 		} lws_end_foreach_llp(pmi, next);
 
+		lws_pthread_mutex_unlock(&v->lock); /* } vhost lock */
 		break;
 
 	case LWS_CALLBACK_CONFIRM_EXTENSION_OKAY:
@@ -312,9 +335,18 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 		lws_protocol_vh_priv_zalloc(lws_get_vhost(wsi),
 				lws_get_protocol(wsi),
 				sizeof(struct per_vhost_data__lws_mirror));
+		v = (struct per_vhost_data__lws_mirror *)
+				lws_protocol_vh_priv_get(lws_get_vhost(wsi),
+							 lws_get_protocol(wsi));
+		lws_pthread_mutex_init(&v->lock);
+		break;
+
+	case LWS_CALLBACK_PROTOCOL_DESTROY:
+		lws_pthread_mutex_destroy(&v->lock);
 		break;
 
 	case LWS_CALLBACK_SERVER_WRITEABLE:
+		lws_pthread_mutex_lock(&pss->mi->lock); /* instance lock { */
 		oldest_tail = lws_ring_get_oldest_tail(pss->mi->ring);
 		update_worst = oldest_tail == pss->tail;
 		sent_something = 0;
@@ -339,7 +371,7 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 			if (n < 0) {
 				lwsl_info("%s: WRITEABLE: %d\n", __func__, n);
 
-				return -1;
+				goto bail2;
 			}
 			sent_something = 1;
 			lws_ring_consume(pss->mi->ring, &pss->tail, NULL, 1);
@@ -352,7 +384,7 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 			lws_callback_on_writable(wsi);
 
 		if (!sent_something || !update_worst)
-			break;
+			goto done1;
 
 		/*
 		 * We are no longer holding the oldest tail (since we sent
@@ -365,15 +397,24 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 		 * all the tails, now we used some up we may have
 		 * changed the oldest fifo position and made some space.
 		 */
-		mirror_update_worst_tail(pss->mi);
+		__mirror_update_worst_tail(pss->mi);
+
+done1:
+		lws_pthread_mutex_unlock(&pss->mi->lock); /* } instance lock */
 		break;
 
+bail2:
+		lws_pthread_mutex_unlock(&pss->mi->lock); /* } instance lock */
+
+		return -1;
+
 	case LWS_CALLBACK_RECEIVE:
-		n = lws_ring_get_count_free_elements(pss->mi->ring);
+		lws_pthread_mutex_lock(&pss->mi->lock); /* mi lock { */
+		n = (int)lws_ring_get_count_free_elements(pss->mi->ring);
 		if (!n) {
 			lwsl_notice("dropping!\n");
 			if (pss->mi->rx_enabled)
-				mirror_rxflow_instance(pss->mi, 0);
+				__mirror_rxflow_instance(pss->mi, 0);
 			goto req_writable;
 		}
 
@@ -381,24 +422,32 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 		amsg.len = len;
 		if (!amsg.payload) {
 			lwsl_notice("OOM: dropping\n");
-			break;
+			goto done2;
 		}
 
 		memcpy((char *)amsg.payload + LWS_PRE, in, len);
 		if (!lws_ring_insert(pss->mi->ring, &amsg, 1)) {
-			mirror_destroy_message(&amsg);
+			__mirror_destroy_message(&amsg);
 			lwsl_notice("dropping!\n");
 			if (pss->mi->rx_enabled)
-				mirror_rxflow_instance(pss->mi, 0);
+				__mirror_rxflow_instance(pss->mi, 0);
 			goto req_writable;
 		}
 
 		if (pss->mi->rx_enabled &&
-		    lws_ring_get_count_free_elements(pss->mi->ring) < RXFLOW_MIN)
-			mirror_rxflow_instance(pss->mi, 0);
+		    lws_ring_get_count_free_elements(pss->mi->ring) <
+								    RXFLOW_MIN)
+			__mirror_rxflow_instance(pss->mi, 0);
 
 req_writable:
-		mirror_callback_all_in_mi_on_writable(pss->mi);
+		__mirror_callback_all_in_mi_on_writable(pss->mi);
+
+done2:
+		lws_pthread_mutex_unlock(&pss->mi->lock); /* } mi lock */
+		break;
+
+	case LWS_CALLBACK_EVENT_WAIT_CANCELLED:
+		lwsl_info("LWS_CALLBACK_EVENT_WAIT_CANCELLED\n");
 		break;
 
 	default:
@@ -412,7 +461,7 @@ req_writable:
 		"lws-mirror-protocol", \
 		callback_lws_mirror, \
 		sizeof(struct per_session_data__lws_mirror), \
-		128, /* rx buf size must be >= permessage-deflate rx size */ \
+		4096, /* rx buf size must be >= permessage-deflate rx size */ \
 		0, NULL, 0 \
 	}
 
@@ -433,7 +482,7 @@ init_protocol_lws_mirror(struct lws_context *context,
 	}
 
 	c->protocols = protocols;
-	c->count_protocols = ARRAY_SIZE(protocols);
+	c->count_protocols = LWS_ARRAY_SIZE(protocols);
 	c->extensions = NULL;
 	c->count_extensions = 0;
 
