@@ -86,28 +86,6 @@ lws_buf(uint8_t **p, void *s, uint32_t len)
 	return 0;
 }
 
-
-void
-explicit_bzero(void *p, size_t len)
-{
-	volatile uint8_t *vp = p;
-
-	while (len--)
-		*vp++ = 0;
-}
-
-int
-lws_timingsafe_bcmp(const void *a, const void *b, uint32_t len)
-{
-	const uint8_t *pa = a, *pb = b;
-	uint8_t sum = 0;
-
-	while (len--)
-		sum |= (*pa++ ^ *pb++);
-
-	return sum;
-}
-
 void
 write_task(struct per_session_data__sshd *pss, struct lws_ssh_channel *ch,
 	   int task)
@@ -402,7 +380,7 @@ lws_kex_destroy(struct per_session_data__sshd *pss)
 		pss->kex->I_S = NULL;
 	}
 
-	explicit_bzero(pss->kex, sizeof(*pss->kex));
+	lws_explicit_bzero(pss->kex, sizeof(*pss->kex));
 	free(pss->kex);
 	pss->kex = NULL;
 }
@@ -436,11 +414,11 @@ lws_ua_destroy(struct per_session_data__sshd *pss)
 	if (pss->ua->pubkey)
 		ssh_free(pss->ua->pubkey);
 	if (pss->ua->sig) {
-		explicit_bzero(pss->ua->sig, pss->ua->sig_len);
+		lws_explicit_bzero(pss->ua->sig, pss->ua->sig_len);
 		ssh_free(pss->ua->sig);
 	}
 
-	explicit_bzero(pss->ua, sizeof(*pss->ua));
+	lws_explicit_bzero(pss->ua, sizeof(*pss->ua));
 	free(pss->ua);
 	pss->ua = NULL;
 }
@@ -548,7 +526,7 @@ lws_ssh_exec_finish(void *finish_handle, int retcode)
 static int
 lws_ssh_parse_plaintext(struct per_session_data__sshd *pss, uint8_t *p, size_t len)
 {
-	struct lws_genrsa_elements el;
+	struct lws_gencrypto_keyelem e[LWS_GENCRYPTO_RSA_KEYEL_COUNT];
 	struct lws_genrsa_ctx ctx;
 	struct lws_ssh_channel *ch;
 	struct lws_subprotocol_scp *scp;
@@ -1037,12 +1015,14 @@ again:
 
 		case SSHS_DO_UAR_SVC:
 			pss->ua->username = (char *)pss->last_alloc;
+			pss->last_alloc = NULL; /* it was adopted */
 			state_get_string_alloc(pss, SSHS_DO_UAR_PUBLICKEY);
 			/* destroyed with UA struct */
 			break;
 
 		case SSHS_DO_UAR_PUBLICKEY:
 			pss->ua->service = (char *)pss->last_alloc;
+			pss->last_alloc = NULL; /* it was adopted */
 
 			/* Sect 5, RFC4252
 			 *
@@ -1104,6 +1084,7 @@ again:
 
 		case SSHS_NVC_DO_UAR_ALG:
 			pss->ua->alg = (char *)pss->last_alloc;
+			pss->last_alloc = NULL; /* it was adopted */
 			if (rsa_hash_alg_from_ident(pss->ua->alg) < 0) {
 				lwsl_notice("unknown alg\n");
 				goto ua_fail;
@@ -1114,7 +1095,7 @@ again:
 
 		case SSHS_NVC_DO_UAR_PUBKEY_BLOB:
 			pss->ua->pubkey = pss->last_alloc;
-			pss->last_alloc = NULL;
+			pss->last_alloc = NULL; /* it was adopted */
 			pss->ua->pubkey_len = pss->npos;
 			/*
 			 * RFC4253
@@ -1172,7 +1153,7 @@ again:
 			}
 			lwsl_info("SSHS_DO_UAR_SIG\n");
 			pss->ua->sig = pss->last_alloc;
-			pss->last_alloc = NULL;
+			pss->last_alloc = NULL; /* it was adopted */
 			pss->ua->sig_len = pss->npos;
 			pss->parser_state = SSHS_MSG_EAT_PADDING;
 
@@ -1247,19 +1228,21 @@ again:
 			 * the E and N factors
 			 */
 
-			memset(&el, 0, sizeof(el));
+			memset(e, 0, sizeof(e));
 			pp = pss->ua->pubkey;
 			m = lws_g32(&pp);
 			pp += m;
 			m = lws_g32(&pp);
-			el.e[JWK_KEY_E].buf = pp;
-			el.e[JWK_KEY_E].len = m;
+			e[LWS_GENCRYPTO_RSA_KEYEL_E].buf = pp;
+			e[LWS_GENCRYPTO_RSA_KEYEL_E].len = m;
 			pp += m;
 			m = lws_g32(&pp);
-			el.e[JWK_KEY_N].buf = pp;
-			el.e[JWK_KEY_N].len = m;
+			e[LWS_GENCRYPTO_RSA_KEYEL_N].buf = pp;
+			e[LWS_GENCRYPTO_RSA_KEYEL_N].len = m;
 
-			if (lws_genrsa_create(&ctx, &el))
+			if (lws_genrsa_create(&ctx, e, pss->vhd->context,
+					      LGRSAM_PKCS1_1_5,
+					      LWS_GENHASH_TYPE_UNKNOWN))
 				goto ua_fail;
 
 			/*
@@ -1354,6 +1337,7 @@ again:
 
 		case SSHS_NVC_DISCONNECT_DESC:
 			pss->disconnect_desc = (char *)pss->last_alloc;
+			pss->last_alloc = NULL; /* it was adopted */
 			state_get_string(pss, SSHS_NVC_DISCONNECT_LANG);
 			break;
 
@@ -1520,12 +1504,13 @@ again:
 		case SSHS_NVC_CHRQ_MODES:
 			/* modes is a stream of byte-pairs, not a string */
 			pss->args.pty.modes = (char *)pss->last_alloc;
+			pss->last_alloc = NULL; /* it was adopted */
 			pss->args.pty.modes_len = pss->npos;
 			n = 0;
 			if (pss->vhd->ops && pss->vhd->ops->pty_req)
 				n = pss->vhd->ops->pty_req(pss->ch_temp->priv,
 							&pss->args.pty);
-			ssh_free_set_NULL(pss->last_alloc);
+			ssh_free_set_NULL(pss->args.pty.modes);
 			if (n)
 				goto chrq_fail;
 			if (pss->rq_want_reply)
